@@ -5,7 +5,7 @@
  *  David Corcoran <corcoran@linuxnet.com>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: pcsclite.h.in 2375 2007-02-05 13:12:18Z rousseau $
+ * $Id: pcsclite.h.in 2612 2007-08-13 07:12:35Z rousseau $
  */
 
 /**
@@ -108,7 +108,8 @@ extern SCARD_IO_REQUEST g_rgSCardT0Pci, g_rgSCardT1Pci,
 #define SCARD_SCOPE_TERMINAL		0x0001	/**< Scope in terminal */
 #define SCARD_SCOPE_SYSTEM		0x0002	/**< Scope in system */
 
-#define SCARD_PROTOCOL_UNSET		0x0000	/**< protocol not set */
+#define SCARD_PROTOCOL_UNDEFINED	0x0000	/**< protocol not set */
+#define SCARD_PROTOCOL_UNSET SCARD_PROTOCOL_UNDEFINED	/* backward compat */
 #define SCARD_PROTOCOL_T0		0x0001	/**< T=0 active protocol. */
 #define SCARD_PROTOCOL_T1		0x0002	/**< T=1 active protocol. */
 #define SCARD_PROTOCOL_RAW		0x0004	/**< Raw active protocol. */
@@ -156,7 +157,7 @@ extern SCARD_IO_REQUEST g_rgSCardT0Pci, g_rgSCardT1Pci,
 #define INFINITE			0xFFFFFFFF	/**< Infinite timeout */
 #endif
 
-#define PCSCLITE_VERSION_NUMBER		"1.4.3"	/**< Current version */
+#define PCSCLITE_VERSION_NUMBER		"1.4.4"	/**< Current version */
 /** Maximum readers context (a slot is count as a reader) */
 #define PCSCLITE_MAX_READERS_CONTEXTS			16
 
