@@ -4,7 +4,7 @@
  * Copyright (C) 1999
  *  David Corcoran <corcoran@linuxnet.com>
  *
- * $Id: wintypes.h,v 1.6 2007-02-13 20:38:52 rousseau Exp $
+ * $Id: wintypes.h 2537 2007-05-22 15:02:26Z rousseau $
  */
 
 /**
@@ -47,20 +47,10 @@ extern "C"
 	typedef DWORD *LPDWORD;
 	typedef char *LPSTR;
 
-	/* this type is deprecated but still used by old drivers and applications
-	 * You should use LPTSTR instead */
-	typedef char *LPTSTR
-#ifdef __GNUC__
-		/* __attribute__ is a GCC only extension */
-		__attribute__ ((deprecated))
-#endif
-		;
-	typedef const char *LPCTSTR
-#ifdef __GNUC__
-		/* __attribute__ is a GCC only extension */
-		__attribute__ ((deprecated))
-#endif
-		;
+	/* these types were deprecated but still used by old drivers and
+	 * applications. So just declare and use them. */
+	typedef LPSTR LPTSTR;
+	typedef LPCSTR LPCTSTR;
 
 #else
 #include <windows.h>
