@@ -6,7 +6,7 @@
  * Copyright (C) 2004
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: eventhandler.h 2151 2006-09-06 20:02:47Z rousseau $
+ * $Id: eventhandler.h 2544 2007-05-23 14:19:45Z rousseau $
  */
 
 /**
@@ -33,7 +33,7 @@ extern "C"
 		char readerName[MAX_READERNAME];
 		DWORD readerState;
 		LONG readerSharing;
-		DWORD lockState;
+		DWORD dummy;
 
 		UCHAR cardAtr[MAX_ATR_SIZE];
 		DWORD cardAtrLength;
@@ -44,7 +44,6 @@ extern "C"
 	LONG EHInitializeEventStructures(void);
 	LONG EHSpawnEventHandler(PREADER_CONTEXT);
 	LONG EHDestroyEventHandler(PREADER_CONTEXT);
-	void EHSetSharingEvent(PREADER_CONTEXT, DWORD);
 
 #ifdef __cplusplus
 }
