@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: readerfactory.c 2377 2007-02-05 13:13:56Z rousseau $
+ * $Id: readerfactory.c 2561 2007-06-13 12:48:12Z rousseau $
  */
 
 /**
@@ -164,12 +164,13 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		sizeof((sReadersContexts[dwContext])->lpcDevice));
 	(sReadersContexts[dwContext])->dwVersion = 0;
 	(sReadersContexts[dwContext])->dwPort = dwPort;
-	(sReadersContexts[dwContext])->mMutex = 0;
+	(sReadersContexts[dwContext])->mMutex = NULL;
 	(sReadersContexts[dwContext])->dwBlockStatus = 0;
 	(sReadersContexts[dwContext])->dwContexts = 0;
 	(sReadersContexts[dwContext])->pthThread = 0;
 	(sReadersContexts[dwContext])->dwLockId = 0;
-	(sReadersContexts[dwContext])->vHandle = 0;
+	(sReadersContexts[dwContext])->LockCount = 0;
+	(sReadersContexts[dwContext])->vHandle = NULL;
 	(sReadersContexts[dwContext])->pdwFeeds = NULL;
 	(sReadersContexts[dwContext])->pdwMutex = NULL;
 	(sReadersContexts[dwContext])->dwIdentity =
@@ -204,7 +205,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		if (rv == IFD_SUCCESS && dwGetSize == 1 && ucThread[0] == 1)
 		{
 			Log1(PCSC_LOG_INFO, "Driver is thread safe");
-			(sReadersContexts[dwContext])->mMutex = 0;
+			(sReadersContexts[dwContext])->mMutex = NULL;
 			(sReadersContexts[dwContext])->pdwMutex = NULL;
 		}
 		else
@@ -252,7 +253,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 
 		(sReadersContexts[dwContext])->dwVersion = 0;
 		(sReadersContexts[dwContext])->dwPort = 0;
-		(sReadersContexts[dwContext])->vHandle = 0;
+		(sReadersContexts[dwContext])->vHandle = NULL;
 		(sReadersContexts[dwContext])->readerState = NULL;
 		(sReadersContexts[dwContext])->dwIdentity = 0;
 
@@ -386,6 +387,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		(sReadersContexts[dwContextB])->dwBlockStatus = 0;
 		(sReadersContexts[dwContextB])->dwContexts = 0;
 		(sReadersContexts[dwContextB])->dwLockId = 0;
+		(sReadersContexts[dwContextB])->LockCount = 0;
 		(sReadersContexts[dwContextB])->readerState = NULL;
 		(sReadersContexts[dwContextB])->dwIdentity =
 			(dwContextB + 1) << (sizeof(DWORD) / 2) * 8;
@@ -428,7 +430,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 
 			(sReadersContexts[dwContextB])->dwVersion = 0;
 			(sReadersContexts[dwContextB])->dwPort = 0;
-			(sReadersContexts[dwContextB])->vHandle = 0;
+			(sReadersContexts[dwContextB])->vHandle = NULL;
 			(sReadersContexts[dwContextB])->readerState = NULL;
 			(sReadersContexts[dwContextB])->dwIdentity = 0;
 
@@ -527,12 +529,13 @@ LONG RFRemoveReader(LPSTR lpcReader, DWORD dwPort)
 		sContext->lpcDevice[0] = 0;
 		sContext->dwVersion = 0;
 		sContext->dwPort = 0;
-		sContext->mMutex = 0;
+		sContext->mMutex = NULL;
 		sContext->dwBlockStatus = 0;
 		sContext->dwContexts = 0;
 		sContext->dwSlot = 0;
 		sContext->dwLockId = 0;
-		sContext->vHandle = 0;
+		sContext->LockCount = 0;
+		sContext->vHandle = NULL;
 		sContext->dwIdentity = 0;
 		sContext->readerState = NULL;
 
@@ -819,7 +822,7 @@ LONG RFLoadReader(PREADER_CONTEXT rContext)
 {
 	if (rContext->vHandle != 0)
 	{
-		Log1(PCSC_LOG_ERROR, "Warning library pointer not NULL");
+		Log1(PCSC_LOG_INFO, "Warning library pointer not NULL");
 		/*
 		 * Another reader exists with this library loaded
 		 */
@@ -1023,7 +1026,7 @@ LONG RFUnloadReader(PREADER_CONTEXT rContext)
 		DYN_CloseLibrary(&rContext->vHandle);
 	}
 
-	rContext->vHandle = 0;
+	rContext->vHandle = NULL;
 
 	return SCARD_S_SUCCESS;
 }
@@ -1053,7 +1056,7 @@ LONG RFLockSharing(DWORD hCard)
 
 	if (RFCheckSharing(hCard) == SCARD_S_SUCCESS)
 	{
-		EHSetSharingEvent(rContext, 1);
+		rContext->LockCount += 1;
 		rContext->dwLockId = hCard;
 	}
 	else
@@ -1075,8 +1078,10 @@ LONG RFUnlockSharing(DWORD hCard)
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	EHSetSharingEvent(rContext, 0);
-	rContext->dwLockId = 0;
+	if (rContext->LockCount > 0)
+		rContext->LockCount -= 1;
+	if (0 == rContext->LockCount)
+		rContext->dwLockId = 0;
 
 	return SCARD_S_SUCCESS;
 }
@@ -1303,6 +1308,13 @@ LONG RFSetReaderEventState(PREADER_CONTEXT rContext, DWORD dwEvent)
 			rContext->psHandles[i].dwEventStatus = dwEvent;
 	}
 
+	if (SCARD_REMOVED == dwEvent)
+	{
+		/* unlock the card */
+		rContext->dwLockId = 0;
+		rContext->LockCount = 0;
+	}
+
 	return SCARD_S_SUCCESS;
 }
 
