@@ -6,7 +6,7 @@
  * Copyright (C) 2004-2006
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: testpcsc.c 2320 2007-01-09 15:28:13Z rousseau $
+ * $Id: testpcsc.c 2541 2007-05-23 07:34:42Z rousseau $
  */
 
 /**
@@ -27,6 +27,7 @@
 #define DONT_PANIC 1
 #define RED_PRINTF_FORMAT "\33[01;31m%s\33[0m\n"
 
+void test_rv(int rv, SCARDCONTEXT hContext, int dont_panic);
 void test_rv(int rv, SCARDCONTEXT hContext, int dont_panic)
 {
 	if (rv != SCARD_S_SUCCESS)
@@ -108,6 +109,7 @@ int main(int argc, char **argv)
 		while (mszGroups[++i] != 0) ;
 	}
 
+wait_for_card_again:
 	printf("Testing SCardListReaders\t: ");
 
 	mszGroups = 0;
@@ -153,6 +155,11 @@ int main(int argc, char **argv)
 	fflush(stdout);
 	rv = SCardGetStatusChange(hContext, INFINITE, rgReaderStates, 1);
 	test_rv(rv, hContext, PANIC);
+	if (SCARD_STATE_EMPTY == rgReaderStates[0].dwEventState)
+	{
+		printf("\nA reader has been connected/disconnected\n");
+		goto wait_for_card_again;
+	}
 
 	printf("Testing SCardConnect\t\t: ");
 	rv = SCardConnect(hContext, &mszReaders[iList[iReader]],
