@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_clnt.c 2377 2007-02-05 13:13:56Z rousseau $
+ * $Id: winscard_clnt.c 2569 2007-06-19 19:10:19Z rousseau $
  */
 
 /**
@@ -25,25 +25,24 @@
 #include <unistd.h>
 #include <sys/un.h>
 #include <errno.h>
+#include <stddef.h>
 
 #include "misc.h"
 #include "pcscd.h"
 #include "winscard.h"
 #include "debug.h"
 #include "thread_generic.h"
+#include "strlcpycat.h"
 
 #include "readerfactory.h"
 #include "eventhandler.h"
 #include "sys_generic.h"
 #include "winscard_msg.h"
+#include "utils.h"
 
 /** used for backward compatibility */
 #define SCARD_PROTOCOL_ANY_OLD	0x1000
 
-#ifndef min
-#define min(a,b) (((a) < (b)) ? (a) : (b))
-#endif
-
 #ifndef TRUE
 #define TRUE 1
 #define FALSE 0
@@ -59,9 +58,10 @@
 struct timeval profile_time_start;
 FILE *fd;
 char profile_tty;
+char fct_name[100];
 
 #define PROFILE_START profile_start(__FUNCTION__);
-#define PROFILE_END profile_end(__FUNCTION__);
+#define PROFILE_END(rv) profile_end(__FUNCTION__, rv);
 
 static void profile_start(const char *f)
 {
@@ -88,6 +88,13 @@ static void profile_start(const char *f)
 			profile_tty = FALSE;
 	}
 
+	/* PROFILE_END was not called before? */
+	if (profile_tty && fct_name[0])
+		printf("\33[01;34m WARNING: %s starts before %s finishes\33[0m\n",
+			f, fct_name);
+
+	strlcpy(fct_name, f, sizeof(fct_name));
+
 	gettimeofday(&profile_time_start, NULL);
 } /* profile_start */
 
@@ -107,7 +114,7 @@ static long int time_sub(struct timeval *a, struct timeval *b)
 } /* time_sub */
 
 
-static void profile_end(const char *f)
+static void profile_end(const char *f, LONG rv)
 {
 	struct timeval profile_time_end;
 	long d;
@@ -116,14 +123,34 @@ static void profile_end(const char *f)
 	d = time_sub(&profile_time_end, &profile_time_start);
 
 	if (profile_tty)
-		fprintf(stderr, "\33[01;31mRESULT %s \33[35m%ld\33[0m\n", f, d);
+	{
+		if (fct_name[0])
+		{
+			if (strncmp(fct_name, f, sizeof(fct_name)))
+				printf("\33[01;34m WARNING: %s ends before %s\33[0m\n",
+						f, fct_name);
+		}
+		else
+			printf("\33[01;34m WARNING: %s ends but we lost its start\33[0m\n",
+				f);
+
+		/* allow to detect missing PROFILE_END calls */
+		fct_name[0] = '\0';
+
+		if (rv != SCARD_S_SUCCESS)
+			fprintf(stderr,
+				"\33[01;31mRESULT %s \33[35m%ld \33[34m0x%08lX %s\33[0m\n",
+				f, d, rv, pcsc_stringify_error(rv));
+		else
+			fprintf(stderr, "\33[01;31mRESULT %s \33[35m%ld\33[0m\n", f, d);
+	}
 	fprintf(fd, "%s %ld\n", f, d);
 	fflush(fd);
 } /* profile_end */
 
 #else
 #define PROFILE_START
-#define PROFILE_END
+#define PROFILE_END(rv)
 #endif
 
 /**
@@ -157,6 +184,13 @@ static struct _psContextMap
  */
 static short isExecuted = 0;
 
+
+/**
+ * creation time of pcscd PCSCLITE_PUBSHM_FILE file
+ */
+static time_t daemon_ctime = 0;
+static pid_t daemon_pid = 0;
+
 /**
  * Memory mapped address used to read status information about the readers.
  * Each element in the vector \ref readerStates makes references to a part of
@@ -187,8 +221,9 @@ static LONG SCardAddContext(SCARDCONTEXT, DWORD);
 static LONG SCardGetContextIndice(SCARDCONTEXT);
 static LONG SCardGetContextIndiceTH(SCARDCONTEXT);
 static LONG SCardRemoveContext(SCARDCONTEXT);
+static LONG SCardCleanContext(LONG indice);
 
-static LONG SCardAddHandle(SCARDHANDLE, DWORD, LPSTR);
+static LONG SCardAddHandle(SCARDHANDLE, DWORD, LPCSTR);
 static LONG SCardGetIndicesFromHandle(SCARDHANDLE, PDWORD, PDWORD);
 static LONG SCardGetIndicesFromHandleTH(SCARDHANDLE, PDWORD, PDWORD);
 static LONG SCardRemoveHandle(SCARDHANDLE);
@@ -196,13 +231,31 @@ static LONG SCardRemoveHandle(SCARDHANDLE);
 static LONG SCardGetSetAttrib(SCARDHANDLE hCard, int command, DWORD dwAttrId,
 	LPBYTE pbAttr, LPDWORD pcbAttrLen);
 
-static LONG SCardCheckDaemonAvailability(void);
+void DESTRUCTOR SCardUnload(void);
 
 /*
  * Thread safety functions
  */
-inline static LONG SCardLockThread(void);
-inline static LONG SCardUnlockThread(void);
+/**
+ * @brief This function locks a mutex so another thread must wait to use this
+ * function.
+ *
+ * Wrapper to the function SYS_MutexLock().
+ */
+inline static LONG SCardLockThread(void)
+{
+	return SYS_MutexLock(&clientMutex);
+}
+
+/**
+ * @brief This function unlocks a mutex so another thread may use the client.
+ *
+ * Wrapper to the function SYS_MutexUnLock().
+ */
+inline static LONG SCardUnlockThread(void)
+{
+	return SYS_MutexUnLock(&clientMutex);
+}
 
 static LONG SCardEstablishContextTH(DWORD, LPCVOID, LPCVOID, LPSCARDCONTEXT);
 
@@ -245,12 +298,16 @@ LONG SCardEstablishContext(DWORD dwScope, LPCVOID pvReserved1,
 
 	PROFILE_START
 
+	/* Check if the server is running */
+	if (SCardCheckDaemonAvailability() != SCARD_S_SUCCESS)
+		return SCARD_E_NO_SERVICE;
+
 	SCardLockThread();
 	rv = SCardEstablishContextTH(dwScope, pvReserved1,
 		pvReserved2, phContext);
 	SCardUnlockThread();
 
-	PROFILE_END
+	PROFILE_END(rv)
 
 	return rv;
 }
@@ -294,10 +351,6 @@ static LONG SCardEstablishContextTH(DWORD dwScope, LPCVOID pvReserved1,
 	else
 		*phContext = 0;
 
-	/* Check if the server is running */
-	if (SCardCheckDaemonAvailability() != SCARD_S_SUCCESS)
-		return SCARD_E_NO_SERVICE;
-
 	/*
 	 * Do this only once:
 	 * - Initialize debug of need.
@@ -325,6 +378,11 @@ static LONG SCardEstablishContextTH(DWORD dwScope, LPCVOID pvReserved1,
 			return SCARD_E_NO_SERVICE;
 		}
 
+		/* close on exec so that child processes do not inherits the file
+		 * descriptor. The child process will call SCardEstablishContext()
+		 * if needed. */
+		fcntl(mapAddr, F_SETFD, FD_CLOEXEC);
+
 		pageSize = SYS_GetPageSize();
 
 		/*
@@ -393,7 +451,6 @@ static LONG SCardEstablishContextTH(DWORD dwScope, LPCVOID pvReserved1,
 	}
 
 	{	/* exchange client/server protocol versions */
-		sharedSegmentMsg msgStruct;
 		version_struct *veStr;
 
 		memset(&msgStruct, 0, sizeof(msgStruct));
@@ -442,7 +499,7 @@ static LONG SCardEstablishContextTH(DWORD dwScope, LPCVOID pvReserved1,
 	 */
 	scEstablishStruct.dwScope = dwScope;
 	scEstablishStruct.phContext = 0;
-	scEstablishStruct.rv = 0;
+	scEstablishStruct.rv = SCARD_S_SUCCESS;
 
 	rv = WrapSHMWrite(SCARD_ESTABLISH_CONTEXT, dwClientID,
 		sizeof(scEstablishStruct), PCSCLITE_MCLIENT_ATTEMPTS,
@@ -514,7 +571,7 @@ LONG SCardReleaseContext(SCARDCONTEXT hContext)
 	SYS_MutexLock(psContextMap[dwContextIndex].mMutex);
 
 	scReleaseStruct.hContext = hContext;
-	scReleaseStruct.rv = 0;
+	scReleaseStruct.rv = SCARD_S_SUCCESS;
 
 	rv = WrapSHMWrite(SCARD_RELEASE_CONTEXT, psContextMap[dwContextIndex].dwClientID,
 			  sizeof(scReleaseStruct),
@@ -547,7 +604,7 @@ LONG SCardReleaseContext(SCARDCONTEXT hContext)
 	SCardRemoveContext(hContext);
 	SCardUnlockThread();
 
-	PROFILE_END
+	PROFILE_END(scReleaseStruct.rv)
 
 	return scReleaseStruct.rv;
 }
@@ -676,8 +733,9 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 	scConnectStruct.hContext = hContext;
 	scConnectStruct.dwShareMode = dwShareMode;
 	scConnectStruct.dwPreferredProtocols = dwPreferredProtocols;
-	scConnectStruct.phCard = *phCard;
-	scConnectStruct.pdwActiveProtocol = *pdwActiveProtocol;
+	scConnectStruct.phCard = 0;
+	scConnectStruct.pdwActiveProtocol = 0;
+	scConnectStruct.rv = SCARD_S_SUCCESS;
 
 	rv = WrapSHMWrite(SCARD_CONNECT, psContextMap[dwContextIndex].dwClientID,
 		sizeof(scConnectStruct),
@@ -710,17 +768,17 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 		/*
 		 * Keep track of the handle locally
 		 */
-		rv = SCardAddHandle(*phCard, dwContextIndex, (LPSTR) szReader);
+		rv = SCardAddHandle(*phCard, dwContextIndex, szReader);
 		SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-		PROFILE_END
+		PROFILE_END(rv)
 
 		return rv;
 	}
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(scConnectStruct.rv)
 
 	return scConnectStruct.rv;
 }
@@ -852,40 +910,44 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 		return SCARD_E_READER_UNAVAILABLE;
 	}
 
-	scReconnectStruct.hCard = hCard;
-	scReconnectStruct.dwShareMode = dwShareMode;
-	scReconnectStruct.dwPreferredProtocols = dwPreferredProtocols;
-	scReconnectStruct.dwInitialization = dwInitialization;
-	scReconnectStruct.pdwActiveProtocol = *pdwActiveProtocol;
-
-	rv = WrapSHMWrite(SCARD_RECONNECT, psContextMap[dwContextIndex].dwClientID,
-		sizeof(scReconnectStruct),
-		PCSCLITE_CLIENT_ATTEMPTS, (void *) &scReconnectStruct);
-
-	if (rv == -1)
+	do
 	{
-		SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
-		return SCARD_E_NO_SERVICE;
-	}
+		scReconnectStruct.hCard = hCard;
+		scReconnectStruct.dwShareMode = dwShareMode;
+		scReconnectStruct.dwPreferredProtocols = dwPreferredProtocols;
+		scReconnectStruct.dwInitialization = dwInitialization;
+		scReconnectStruct.pdwActiveProtocol = *pdwActiveProtocol;
+		scReconnectStruct.rv = SCARD_S_SUCCESS;
 
-	/*
-	 * Read a message from the server
-	 */
-	rv = SHMClientRead(&msgStruct, psContextMap[dwContextIndex].dwClientID, PCSCLITE_CLIENT_ATTEMPTS);
+		rv = WrapSHMWrite(SCARD_RECONNECT, psContextMap[dwContextIndex].dwClientID,
+			sizeof(scReconnectStruct),
+			PCSCLITE_CLIENT_ATTEMPTS, (void *) &scReconnectStruct);
 
-	memcpy(&scReconnectStruct, &msgStruct.data, sizeof(scReconnectStruct));
+		if (rv == -1)
+		{
+			SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
+			return SCARD_E_NO_SERVICE;
+		}
 
-	if (rv == -1)
-	{
-		SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
-		return SCARD_F_COMM_ERROR;
-	}
+		/*
+		 * Read a message from the server
+		 */
+		rv = SHMClientRead(&msgStruct, psContextMap[dwContextIndex].dwClientID, PCSCLITE_CLIENT_ATTEMPTS);
+
+		memcpy(&scReconnectStruct, &msgStruct.data, sizeof(scReconnectStruct));
+
+		if (rv == -1)
+		{
+			SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
+			return SCARD_F_COMM_ERROR;
+		}
+	} while (SCARD_E_SHARING_VIOLATION == scReconnectStruct.rv);
 
 	*pdwActiveProtocol = scReconnectStruct.pdwActiveProtocol;
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(scReconnectStruct.rv)
 
 	return scReconnectStruct.rv;
 }
@@ -952,6 +1014,7 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 
 	scDisconnectStruct.hCard = hCard;
 	scDisconnectStruct.dwDisposition = dwDisposition;
+	scDisconnectStruct.rv = SCARD_S_SUCCESS;
 
 	rv = WrapSHMWrite(SCARD_DISCONNECT, psContextMap[dwContextIndex].dwClientID,
 		sizeof(scDisconnectStruct),
@@ -981,7 +1044,7 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(scDisconnectStruct.rv)
 
 	return scDisconnectStruct.rv;
 }
@@ -1058,6 +1121,7 @@ LONG SCardBeginTransaction(SCARDHANDLE hCard)
 	}
 
 	scBeginStruct.hCard = hCard;
+	scBeginStruct.rv = SCARD_S_SUCCESS;
 
 	/*
 	 * Query the server every so often until the sharing violation ends
@@ -1097,7 +1161,7 @@ LONG SCardBeginTransaction(SCARDHANDLE hCard)
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(scBeginStruct.rv);
 
 	return scBeginStruct.rv;
 }
@@ -1193,6 +1257,7 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 
 	scEndStruct.hCard = hCard;
 	scEndStruct.dwDisposition = dwDisposition;
+	scEndStruct.rv = SCARD_S_SUCCESS;
 
 	rv = WrapSHMWrite(SCARD_END_TRANSACTION, psContextMap[dwContextIndex].dwClientID,
 		sizeof(scEndStruct),
@@ -1225,7 +1290,7 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(scEndStruct.rv)
 
 	return scEndStruct.rv;
 }
@@ -1300,7 +1365,7 @@ LONG SCardCancelTransaction(SCARDHANDLE hCard)
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(scCancelStruct.rv)
 
 	return scCancelStruct.rv;
 }
@@ -1500,7 +1565,7 @@ LONG SCardStatus(SCARDHANDLE hCard, LPSTR mszReaderNames,
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(rv)
 
 	return rv;
 }
@@ -1646,7 +1711,7 @@ LONG SCardGetStatusChange(SCARDCONTEXT hContext, DWORD dwTimeout,
 					 */
 					SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-					PROFILE_END
+					PROFILE_END(SCARD_S_SUCCESS)
 
 					return SCARD_S_SUCCESS;
 				}
@@ -1671,7 +1736,7 @@ LONG SCardGetStatusChange(SCARDCONTEXT hContext, DWORD dwTimeout,
 				{
 					SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-					PROFILE_END
+					PROFILE_END(SCARD_E_TIMEOUT)
 
 					return SCARD_E_TIMEOUT;
 				}
@@ -1735,9 +1800,10 @@ LONG SCardGetStatusChange(SCARDCONTEXT hContext, DWORD dwTimeout,
 
 		if (SCardCheckDaemonAvailability() != SCARD_S_SUCCESS)
 		{
-			SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
+			if (psContextMap[dwContextIndex].mMutex)
+				SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-			PROFILE_END
+			PROFILE_END(SCARD_E_NO_SERVICE)
 
 			return SCARD_E_NO_SERVICE;
 		}
@@ -1821,6 +1887,21 @@ LONG SCardGetStatusChange(SCARDCONTEXT hContext, DWORD dwTimeout,
 				 * Now we check all the Reader States
 				 */
 				dwState = rContext->readerState;
+				{
+					int currentCounter, stateCounter;
+
+					stateCounter = (dwState >> 16) & 0xFFFF;
+					currentCounter = (currReader->dwCurrentState >> 16) & 0xFFFF;
+
+					/* has the event counter changed since the last call? */
+					if (stateCounter != currentCounter)
+						currReader->dwEventState |= SCARD_STATE_CHANGED;
+
+					/* add an event counter in the upper word of dwEventState */
+					currReader->dwEventState =
+						((currReader->dwEventState & 0xffff )
+						| (stateCounter << 16));  
+				}
 
 	/*********** Check if the reader is in the correct state ********/
 				if (dwState & SCARD_UNKNOWN)
@@ -1889,10 +1970,7 @@ LONG SCardGetStatusChange(SCARDCONTEXT hContext, DWORD dwTimeout,
 					/*
 					 * After present the rest are assumed
 					 */
-					if (currReader->dwCurrentState & SCARD_STATE_PRESENT
-						|| currReader->dwCurrentState & SCARD_STATE_ATRMATCH
-						|| currReader->dwCurrentState & SCARD_STATE_EXCLUSIVE
-						|| currReader->dwCurrentState & SCARD_STATE_INUSE)
+					if (currReader->dwCurrentState & SCARD_STATE_PRESENT)
 					{
 						currReader->dwEventState |= SCARD_STATE_CHANGED;
 						dwBreakFlag = 1;
@@ -2052,6 +2130,7 @@ LONG SCardGetStatusChange(SCARDCONTEXT hContext, DWORD dwTimeout,
 			if ((dwTime >= (dwTimeout * 1000)) && (j == 0))
 			{
 				SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
+				PROFILE_END(SCARD_E_TIMEOUT)
 				return SCARD_E_TIMEOUT;
 			}
 		}
@@ -2073,12 +2152,13 @@ LONG SCardGetStatusChange(SCARDCONTEXT hContext, DWORD dwTimeout,
 			BLOCK_STATUS_RESUME)
 	{
 		SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
+		PROFILE_END(SCARD_E_CANCELLED)
 		return SCARD_E_CANCELLED;
 	}
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(SCARD_S_SUCCESS)
 
 	return SCARD_S_SUCCESS;
 }
@@ -2191,7 +2271,15 @@ LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode, LPCVOID pbSendBuffer,
 		scControlStructExtended->dwControlCode = dwControlCode;
 		scControlStructExtended->cbSendLength = cbSendLength;
 		scControlStructExtended->cbRecvLength = cbRecvLength;
-		scControlStructExtended->size = sizeof(*scControlStructExtended) + cbSendLength;
+		scControlStructExtended->pdwBytesReturned = 0;
+		scControlStructExtended->rv = SCARD_S_SUCCESS;
+		/* The size of data to send is the size of
+		 * struct control_struct_extended WITHOUT the data[] field
+		 * plus the effective data[] size
+		 */
+		scControlStructExtended->size = sizeof(*scControlStructExtended)
+			- (sizeof(control_struct_extended) - offsetof(control_struct_extended, data))
+			+ cbSendLength;
 		memcpy(scControlStructExtended->data, pbSendBuffer, cbSendLength);
 
 		rv = WrapSHMWrite(SCARD_CONTROL_EXTENDED,
@@ -2298,7 +2386,7 @@ LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode, LPCVOID pbSendBuffer,
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(rv)
 
 	return rv;
 }
@@ -2386,6 +2474,8 @@ LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode, LPCVOID pbSendBuffer,
 LONG SCardGetAttrib(SCARDHANDLE hCard, DWORD dwAttrId, LPBYTE pbAttr,
 	LPDWORD pcbAttrLen)
 {
+	LONG ret;
+
 	PROFILE_START
 
 	if (NULL == pcbAttrLen)
@@ -2396,10 +2486,12 @@ LONG SCardGetAttrib(SCARDHANDLE hCard, DWORD dwAttrId, LPBYTE pbAttr,
 		/* this variable may not be set by the caller. use a reasonable size */
 		*pcbAttrLen = MAX_BUFFER_SIZE;
 
-	PROFILE_END
-
-	return SCardGetSetAttrib(hCard, SCARD_GET_ATTRIB, dwAttrId, pbAttr,
+	ret = SCardGetSetAttrib(hCard, SCARD_GET_ATTRIB, dwAttrId, pbAttr,
 		pcbAttrLen);
+
+	PROFILE_END(ret)
+
+	return ret;
 }
 
 /**
@@ -2435,22 +2527,24 @@ LONG SCardGetAttrib(SCARDHANDLE hCard, DWORD dwAttrId, LPBYTE pbAttr,
 LONG SCardSetAttrib(SCARDHANDLE hCard, DWORD dwAttrId, LPCBYTE pbAttr,
 	DWORD cbAttrLen)
 {
+	LONG ret;
+
 	PROFILE_START
 
 	if (NULL == pbAttr || 0 == cbAttrLen)
 		return SCARD_E_INVALID_PARAMETER;
 
-	PROFILE_END
-
-	return SCardGetSetAttrib(hCard, SCARD_SET_ATTRIB, dwAttrId, (LPBYTE)pbAttr,
+	ret = SCardGetSetAttrib(hCard, SCARD_SET_ATTRIB, dwAttrId, (LPBYTE)pbAttr,
 		&cbAttrLen);
+
+	PROFILE_END(ret)
+
+	return ret;
 }
 
 static LONG SCardGetSetAttrib(SCARDHANDLE hCard, int command, DWORD dwAttrId,
 	LPBYTE pbAttr, LPDWORD pcbAttrLen)
 {
-	PROFILE_START
-
 	LONG rv;
 	getset_struct scGetSetStruct;
 	sharedSegmentMsg msgStruct;
@@ -2495,6 +2589,7 @@ static LONG SCardGetSetAttrib(SCARDHANDLE hCard, int command, DWORD dwAttrId,
 	scGetSetStruct.dwAttrId = dwAttrId;
 	scGetSetStruct.cbAttrLen = *pcbAttrLen;
 	scGetSetStruct.rv = SCARD_E_NO_SERVICE;
+	memset(scGetSetStruct.pbAttr, 0, sizeof(scGetSetStruct.pbAttr));
 	if (SCARD_SET_ATTRIB == command)
 		memcpy(scGetSetStruct.pbAttr, pbAttr, *pcbAttrLen);
 
@@ -2542,8 +2637,6 @@ static LONG SCardGetSetAttrib(SCARDHANDLE hCard, int command, DWORD dwAttrId,
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
-
 	return scGetSetStruct.rv;
 }
 
@@ -2663,10 +2756,17 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 		scTransmitStructExtended->hCard = hCard;
 		scTransmitStructExtended->cbSendLength = cbSendLength;
 		scTransmitStructExtended->pcbRecvLength = *pcbRecvLength;
-		scTransmitStructExtended->size = sizeof(*scTransmitStructExtended) + cbSendLength;
+		/* The size of data to send is the size of
+		 * struct control_struct_extended WITHOUT the data[] field
+		 * plus the effective data[] size
+		 */
+		scTransmitStructExtended->size = sizeof(*scTransmitStructExtended)
+			- (sizeof(transmit_struct_extended) - offsetof(transmit_struct_extended, data))
+			+ cbSendLength;
 		memcpy(&scTransmitStructExtended->pioSendPci, pioSendPci,
 			sizeof(SCARD_IO_REQUEST));
 		memcpy(scTransmitStructExtended->data, pbSendBuffer, cbSendLength);
+		scTransmitStructExtended->rv = SCARD_S_SUCCESS;
 
 		if (pioRecvPci)
 		{
@@ -2674,7 +2774,10 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 				sizeof(SCARD_IO_REQUEST));
 		}
 		else
+		{
 			scTransmitStructExtended->pioRecvPci.dwProtocol = SCARD_PROTOCOL_ANY;
+			scTransmitStructExtended->pioRecvPci.cbPciLength = sizeof(SCARD_IO_REQUEST);
+		}
 
 		rv = WrapSHMWrite(SCARD_TRANSMIT_EXTENDED,
 			psContextMap[dwContextIndex].dwClientID,
@@ -2747,14 +2850,12 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 		memcpy(&scTransmitStruct.pioSendPci, pioSendPci,
 			sizeof(SCARD_IO_REQUEST));
 		memcpy(scTransmitStruct.pbSendBuffer, pbSendBuffer, cbSendLength);
+		memset(scTransmitStruct.pbSendBuffer+cbSendLength, 0, sizeof(scTransmitStruct.pbSendBuffer)-cbSendLength);
+		memset(scTransmitStruct.pbRecvBuffer, 0, sizeof(scTransmitStruct.pbRecvBuffer));
+		scTransmitStruct.rv = SCARD_S_SUCCESS;
 
-		if (pioRecvPci)
-		{
-			memcpy(&scTransmitStruct.pioRecvPci, pioRecvPci,
-				sizeof(SCARD_IO_REQUEST));
-		}
-		else
-			scTransmitStruct.pioRecvPci.dwProtocol = SCARD_PROTOCOL_ANY;
+		scTransmitStruct.pioRecvPci.dwProtocol = SCARD_PROTOCOL_ANY;
+		scTransmitStruct.pioRecvPci.cbPciLength = sizeof(scTransmitStruct.pioRecvPci);
 
 		rv = WrapSHMWrite(SCARD_TRANSMIT,
 			psContextMap[dwContextIndex].dwClientID, sizeof(scTransmitStruct),
@@ -2805,7 +2906,7 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 		rv = scTransmitStruct.rv;
 	}
 
-	PROFILE_END
+	PROFILE_END(rv)
 
 	return rv;
 }
@@ -2879,6 +2980,7 @@ LONG SCardListReaders(SCARDCONTEXT hContext, LPCSTR mszGroups,
 	{
 		*pcchReaders = dwReadersLen;
 		SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
+		PROFILE_END(SCARD_S_SUCCESS)
 		return SCARD_S_SUCCESS;
 	}
 
@@ -2886,6 +2988,7 @@ LONG SCardListReaders(SCARDCONTEXT hContext, LPCSTR mszGroups,
 	{
 		*pcchReaders = dwReadersLen;
 		SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
+		PROFILE_END(SCARD_E_INSUFFICIENT_BUFFER)
 		return SCARD_E_INSUFFICIENT_BUFFER;
 	}
 
@@ -2907,7 +3010,7 @@ LONG SCardListReaders(SCARDCONTEXT hContext, LPCSTR mszGroups,
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(SCARD_S_SUCCESS)
 
 	return SCARD_S_SUCCESS;
 }
@@ -2982,7 +3085,7 @@ LONG SCardListReaderGroups(SCARDCONTEXT hContext, LPSTR mszGroups,
 
 	SYS_MutexUnLock(psContextMap[dwContextIndex].mMutex);
 
-	PROFILE_END
+	PROFILE_END(rv)
 
 	return rv;
 }
@@ -3031,7 +3134,7 @@ LONG SCardCancel(SCARDCONTEXT hContext)
 	 */
 	psContextMap[dwContextIndex].contextBlockStatus = BLOCK_STATUS_RESUME;
 
-	PROFILE_END
+	PROFILE_END(SCARD_S_SUCCESS)
 
 	return SCARD_S_SUCCESS;
 }
@@ -3068,6 +3171,10 @@ LONG SCardIsValidContext(SCARDCONTEXT hContext)
 
 	rv = SCARD_S_SUCCESS;
 
+	/* Check if the _same_ server is running */
+	if (SCardCheckDaemonAvailability() != SCARD_S_SUCCESS)
+		return SCARD_E_INVALID_HANDLE;
+
 	/*
 	 * Make sure this context has been opened
 	 */
@@ -3075,7 +3182,7 @@ LONG SCardIsValidContext(SCARDCONTEXT hContext)
 	if (dwContextIndex == -1)
 		rv = SCARD_E_INVALID_HANDLE;
 
-	PROFILE_END
+	PROFILE_END(rv)
 
 	return rv;
 }
@@ -3185,28 +3292,31 @@ static LONG SCardRemoveContext(SCARDCONTEXT hContext)
 	if (retIndice == -1)
 		return SCARD_E_INVALID_HANDLE;
 	else
+		return SCardCleanContext(retIndice);
+}
+
+static LONG SCardCleanContext(LONG indice)
+{
+	int i;
+
+	psContextMap[indice].hContext = 0;
+	SHMClientCloseSession(psContextMap[indice].dwClientID);
+	psContextMap[indice].dwClientID = 0;
+	free(psContextMap[indice].mMutex);
+	psContextMap[indice].mMutex = NULL;
+	psContextMap[indice].contextBlockStatus = BLOCK_STATUS_RESUME;
+
+	for (i = 0; i < PCSCLITE_MAX_APPLICATION_CONTEXT_CHANNELS; i++)
 	{
-		int i;
-
-		psContextMap[retIndice].hContext = 0;
-		SHMClientCloseSession(psContextMap[retIndice].dwClientID);
-		psContextMap[retIndice].dwClientID = 0;
-		free(psContextMap[retIndice].mMutex);
-		psContextMap[retIndice].mMutex = NULL;
-		psContextMap[retIndice].contextBlockStatus = BLOCK_STATUS_RESUME;
-
-		for (i = 0; i < PCSCLITE_MAX_APPLICATION_CONTEXT_CHANNELS; i++)
-		{
-			/*
-			 * Reset the \c hCard structs to zero
-			 */
-			psContextMap[retIndice].psChannelMap[i].hCard = 0;
-			free(psContextMap[retIndice].psChannelMap[i].readerName);
-			psContextMap[retIndice].psChannelMap[i].readerName = NULL;
-		}
-
-		return SCARD_S_SUCCESS;
+		/*
+		 * Reset the \c hCard structs to zero
+		 */
+		psContextMap[indice].psChannelMap[i].hCard = 0;
+		free(psContextMap[indice].psChannelMap[i].readerName);
+		psContextMap[indice].psChannelMap[i].readerName = NULL;
 	}
+
+	return SCARD_S_SUCCESS;
 }
 
 /*
@@ -3214,7 +3324,7 @@ static LONG SCardRemoveContext(SCARDCONTEXT hContext)
  */
 
 static LONG SCardAddHandle(SCARDHANDLE hCard, DWORD dwContextIndex,
-	LPSTR readerName)
+	LPCSTR readerName)
 {
 	int i;
 
@@ -3289,27 +3399,6 @@ static LONG SCardGetIndicesFromHandleTH(SCARDHANDLE hCard, PDWORD pdwContextIndi
 	return -1;
 }
 
-/**
- * @brief This function locks a mutex so another thread must wait to use this
- * function.
- *
- * Wrapper to the function SYS_MutexLock().
- */
-inline static LONG SCardLockThread(void)
-{
-	return SYS_MutexLock(&clientMutex);
-}
-
-/**
- * @brief This function unlocks a mutex so another thread may use the client.
- *
- * Wrapper to the function SYS_MutexUnLock().
- */
-inline static LONG SCardUnlockThread(void)
-{
-	return SYS_MutexUnLock(&clientMutex);
-}
-
 /**
  * @brief Checks if the Server is running.
  *
@@ -3317,7 +3406,7 @@ inline static LONG SCardUnlockThread(void)
  * @retval SCARD_S_SUCCESS Server is running (\ref SCARD_S_SUCCESS)
  * @retval SCARD_E_NO_SERVICE Server is not running (\ref SCARD_E_NO_SERVICE)
  */
-static LONG SCardCheckDaemonAvailability(void)
+LONG SCardCheckDaemonAvailability(void)
 {
 	LONG rv;
 	struct stat statBuffer;
@@ -3330,6 +3419,50 @@ static LONG SCardCheckDaemonAvailability(void)
 		return SCARD_E_NO_SERVICE;
 	}
 
+	if (daemon_ctime)
+	{
+		/* when the _first_ reader is connected the ctime changes
+		 * I don't know why yet */
+		if (statBuffer.st_ctime > daemon_ctime)
+		{
+			pid_t new_pid;
+
+			new_pid = GetDaemonPid();
+
+			/* so we also check the daemon pid to be sure it is a new pcscd */
+			if (new_pid != daemon_pid)
+			{
+				int i;
+
+				Log1(PCSC_LOG_ERROR, "PCSC restarted");
+
+				/* invalid all handles */
+				SCardLockThread();
+
+				for (i = 0; i < PCSCLITE_MAX_APPLICATION_CONTEXTS; i++)
+					if (psContextMap[i].hContext)
+						SCardCleanContext(i);
+
+				SCardUnlockThread();
+
+				/* reset pcscd status */
+				daemon_ctime = 0;
+
+				/* reset the lib */
+				SCardUnload();
+
+				return SCARD_E_NO_SERVICE;
+			}
+
+			daemon_ctime = statBuffer.st_ctime;
+		}
+	}
+	else
+	{
+		daemon_ctime = statBuffer.st_ctime;
+		daemon_pid = GetDaemonPid();
+	}
+
 	return SCARD_S_SUCCESS;
 }
 
