@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludoic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_msg.c 2377 2007-02-05 13:13:56Z rousseau $
+ * $Id: winscard_msg.c 2565 2007-06-17 16:48:47Z rousseau $
  */
 
 /**
@@ -41,6 +41,7 @@
 #include "debug.h"
 #include "winscard_msg.h"
 #include "sys_generic.h"
+#include "utils.h"
 
 /**
  * @brief Wrapper for the SHMMessageReceive() function.
@@ -351,11 +352,30 @@ INTERNAL int SHMMessageReceive(void *buffer_void, size_t buffer_size,
 			}
 		} else if (selret == 0)
 		{
+#ifdef PCSCD
 			/*
 			 * timeout
 			 */
 			retval = -1;
 			break;
+#else
+			/* is the daemon still there? */
+			if (SCardCheckDaemonAvailability() != SCARD_S_SUCCESS)
+			{	
+				/*
+				 * timeout
+				 */
+				retval = -1;
+				break;
+			}
+
+			/* the command is extra slow so we keep waiting */
+			start = time(0);
+
+			/* you need to set the env variable PCSCLITE_DEBUG=0 since this is
+			 * logged on the client side and not on the pcscd side*/
+			Log1(PCSC_LOG_INFO, "Command not yet finished");
+#endif
 		} else
 		{
 			/*
@@ -406,25 +426,30 @@ INTERNAL int WrapSHMWrite(unsigned int command, DWORD dwClientID,
 	msgStruct.user_id = SYS_GetUID();
 	msgStruct.group_id = SYS_GetGID();
 	msgStruct.command = command;
+	msgStruct.dummy = 0;
 	msgStruct.date = time(NULL);
+	memset(msgStruct.key, 0, sizeof(msgStruct.key));
 	if ((SCARD_TRANSMIT_EXTENDED == command)
 		|| (SCARD_CONTROL_EXTENDED == command))
 	{
 		/* first block */
-		memcpy(msgStruct.data, data, PCSCLITE_MAX_MESSAGE_SIZE);
+		if (size > sizeof(msgStruct.data))
+			memcpy(msgStruct.data, data, sizeof(msgStruct.data));
+		else
+		{
+			memcpy(msgStruct.data, data, size);
+			memset(msgStruct.data+size, 0, sizeof(msgStruct.data)-size);
+		}
+			
 		ret = SHMMessageSend(&msgStruct, sizeof(msgStruct), dwClientID,
 			blockAmount);
-		if (ret)
-			return ret;
 
 		/* do not send an empty second block */
-		if (size > PCSCLITE_MAX_MESSAGE_SIZE)
+		if ((0 == ret) && (size > sizeof(msgStruct.data)))
 		{
 			/* second block */
-			ret = SHMMessageSend(data+PCSCLITE_MAX_MESSAGE_SIZE,
-				size-PCSCLITE_MAX_MESSAGE_SIZE, dwClientID, blockAmount);
-			if (ret)
-				return ret;
+			ret = SHMMessageSend(data+sizeof(msgStruct.data),
+				size-sizeof(msgStruct.data), dwClientID, blockAmount);
 		}
 	}
 	else
@@ -434,6 +459,11 @@ INTERNAL int WrapSHMWrite(unsigned int command, DWORD dwClientID,
 		ret = SHMMessageSend(&msgStruct, sizeof(msgStruct), dwClientID,
 			blockAmount);
 	}
+
+	if (SCARD_TRANSMIT == command)
+		/* clean APDU buffer to remove any possible PIN or secret value */
+		memset(msgStruct.data, 0, min(size, sizeof(msgStruct.data)));
+
 	return ret;
 }
 
